# Generated by Django 3.2.5 on 2021-07-20 17:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_secretstore', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='secret',
            table=None,
        ),
        migrations.AlterModelTable(
            name='secretrole',
            table=None,
        ),
        migrations.AlterModelTable(
            name='sessionkey',
            table=None,
        ),
        migrations.AlterModelTable(
            name='userkey',
            table=None,
        ),
    ]
